package nl.sbdeveloper.themeparkplus.util;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.bukkit.Bukkit;
import org.bukkit.plugin.Plugin;
import org.jetbrains.annotations.Nullable;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Objects;

/**
 * License class for SBDevelopment
 *
 * v1.3 - Changed in 03-03-2020
 *
 * @author Stijn [SBDeveloper]
 * @since 23-12-2019
 */
public class License {
    /*
     This file is part of ActionFoto.
     Copyright (c) 2018-2020 SBDevelopment - All Rights Reserved
     Unauthorized copying of this file, via any medium is strictly prohibited
     Proprietary and confidential
     Written by Stijn Bannink <stijnbannink23@gmail.com>, January 2020
    */

    private Plugin plugin;
    private String license;
    private String prefix;

    /**
     * Construct a new license
     *  @param plugin The Main class [Javaplugin]
     * @param prefix The prefix, like TPP or AF
     * @param license The license from the config
     */
    public License(Plugin plugin, String prefix, String license) {
        this.prefix = prefix;
        this.plugin = plugin;
        this.license = license;

        startTimer();
    }

    private void startTimer() {
        Bukkit.getScheduler().runTaskTimerAsynchronously(plugin, () -> {
            if (!validateLicense()) {
                Bukkit.getLogger().severe("[" + prefix + "] License is incorrect!");
            }
        }, 0, 20 * 60 * 60);
    }

    /**
     * Check a license
     *
     * @return true/false
     */
    private boolean validateLicense() {
        String url = "https://sbdplugins.nl/wp-json/lmfwc/v2/licenses/" + this.license;

        @Nullable JsonObject res;
        try {
            res = sendGETRequestJSON(url);
        } catch (IOException e) {
            disable("GET_request_error");
            return false;
        }

        if (res == null) {
            disable("GET_request_error_2");
            return false;
        }

        JsonObject dat = res.get("data").getAsJsonObject();

        int stat = dat.get("status").getAsInt();
        if (stat == 404) {
            disable("status_404_error");
            return false;
        }

        if (dat.get("licenseKey").isJsonNull()) {
            disable("license_null_error");
            return false;
        }

        if (!dat.get("licenseKey").getAsString().split("-")[0].contains(prefix)) {
            disable("prefix_error");
            return false;
        }

        switch(dat.get("status").getAsString()) {
            case "2":
                //activate?
                break;
            case "3":
                //it's good
                break;
            default:
                disable("status_error");
                return false;
        }

        //Not activated? Activate it!
        if (dat.get("timesActivated").isJsonNull() || dat.get("timesActivated").getAsString().equalsIgnoreCase("0")) {
            return activate();
        }

        if (dat.get("expiresAt").isJsonNull()) {
            disable("null_error");
            return false;
        }

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date;
        try {
            date = format.parse(dat.get("expiresAt").getAsString());
        } catch (ParseException e) {
            e.printStackTrace();
            disable("null_error");
            return false;
        }

        if (!(Objects.requireNonNull(date).after(new Date()))) {
            disable("expired_error");
            return false;
        }

        if (!dat.get("ipcheck").getAsBoolean()) {
            disable("ip_error");
            return false;
        }

        if (dat.get("port").isJsonNull()) {
            disable("null_error");
            return false;
        }

        try {
            int por = dat.get("port").getAsInt();
            if (por != Bukkit.getServer().getPort()) {
                disable("port_error");
                return false;
            }
        } catch(NumberFormatException e) {
            disable("null_error");
            return false;
        }

        return true;
    }

    /**
     * Activate the license (private)
     *
     * @return true/false
     */
    private boolean activate() {
        String url = "https://sbdplugins.nl/wp-json/lmfwc/v2/licenses/activate/" + this.license + "?port=" + Bukkit.getServer().getPort();

        @Nullable JsonObject res;
        try {
            res = sendGETRequestJSON(url);
        } catch (IOException e) {
            e.printStackTrace();
            disable("GET_request_error");
            return false;
        }

        if (res == null) {
            disable("GET_request_error_2");
            return false;
        }

        JsonObject dat = res.get("data").getAsJsonObject();

        int stat = dat.get("status").getAsInt();
        if (stat == 404) {
            disable("status_404_error");
            return false;
        }

        if (dat.get("licenseKey").isJsonNull()) {
            disable("license_null_error");
            return false;
        }

        if (!dat.get("licenseKey").getAsString().split("-")[0].contains(prefix)) {
            disable("prefix_error");
            return false;
        }

        switch(dat.get("status").getAsString()) {
            case "2":
                //activate?
                break;
            case "3":
                //it's good
                break;
            default:
                disable("status_error");
                return false;
        }

        //Still not activated? Something is wrong...
        return !dat.get("timesActivated").isJsonNull() && !dat.get("timesActivated").getAsString().equalsIgnoreCase("0");
    }

    /**
     * Disable the plugin (private)
     *
     * @param reason The disabling reason
     */
    private void disable(String reason) {
        Bukkit.getScheduler().runTask(plugin, () -> {
            Bukkit.getLogger().severe("[" + plugin.getName() + "] " + "Stopping plugin because licensing system check failed.");
            Bukkit.getLogger().severe("[" + plugin.getName() + "] " + "Reason: " + reason);
            Bukkit.getPluginManager().disablePlugin(plugin);
        });
    }

    /**
     * Send an GET request with JSONObject response
     *
     * @param uri The URL
     *
     * @return The JSONObject
     * @throws IOException URL errors
     */
    @Nullable
    private JsonObject sendGETRequestJSON(String uri) throws IOException {
        URL url = new URL(uri);
        HttpURLConnection con = (HttpURLConnection) url.openConnection();
        con.setRequestMethod("GET");
        con.setRequestProperty("User-Agent", "Mozilla/5.0");
        String authStringEnc = "Y2tfMGEzNWEzMWE2NzExNmM3NDg2MGEwYTJhNjUxNGVjZjM4NTBmM2JmMDpjc185NmYxZGNlYjI4MWRkZDExOTBjMzQ3ZjJjYzMwMGNjZTIzYWNhODI1";
        con.setRequestProperty("Authorization", "Basic " + authStringEnc);
        int code = con.getResponseCode();
        if (code == 404) {
            disable("404_error");
            return null;
        }
        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuilder response = new StringBuilder();
        while ((inputLine = in.readLine()) != null) {
            response.append(inputLine);
        }
        in.close();

        JsonParser parser = new JsonParser();
        return parser.parse(response.toString()).getAsJsonObject();
    }

}